@@ -64,7 +64,9 @@ enum lwasm_output_e
 	OUTPUT_SREC,		// motorola SREC format
 	OUTPUT_IHEX,		// intel hex format
 	OUTPUT_HEX,			// generic hexadecimal format
-	OUTPUT_LWMOD        // special module format for LW
+	OUTPUT_LWMOD,       // special module format for LW
+	OUTPUT_DRAGON,      // Dragon DOS binary
+	OUTPUT_ABS          // absolute binary block
 };
 
 enum lwasm_flags_e
@@ -113,6 +115,7 @@ enum lwasm_pragmas_e
 	PRAGMA_EMUEXT				= 1 << 26,  // enable emulator extensions
 	PRAGMA_NOOUTPUT             = 1 << 27,  // disable object code output
 	PRAGMA_NOEXPANDCOND         = 1 << 28,  // hide conditionals and skipped output in listings
+	PRAGMA_NOLISTCODE           = 1 << 29,  // hide line in listing even if it generates code
 	PRAGMA_CLEARBIT				= 1 << 31	// reserved to indicate negated pragma flag status
 };
 
@@ -213,6 +216,8 @@ typedef enum
 	E_ORG_NOT_FOUND				= 57,
 	E_COMPLEX_INCOMPLETE        = 58,
 	E_ILL5						= 59,
+	E_INCLUDEBIN_ILL_START      = 60,
+	E_INCLUDEBIN_ILL_LENGTH     = 61,
 	
 	/* warnings must be 1000 or greater */
 
@@ -223,6 +228,9 @@ typedef enum
 	W_OPERAND_SIZE				= 1004
 } lwasm_errorcode_t;
 
+#define NOWARN_NONE             0
+#define NOWARN_IFP1             1
+
 typedef struct lwasm_error_s lwasm_error_t;
 struct lwasm_error_s
 {
@@ -439,6 +447,8 @@ struct asmstate_s
 	int fileerr;						// flags error opening file
 	int exprwidth;						// the bit width of the expression being evaluated
 	int listnofile;						// nonzero to suppress printing file name in listings
+	
+	int nowarn_flags;                   // flags indicating which warnings to suppress
 };
 
 struct symtabe *register_symbol(asmstate_t *as, line_t *cl, char *sym, lw_expr_t value, int flags);
