@@ -32,6 +32,7 @@ Contains the code for actually outputting the assembled code
 #include <lw_expr.h>
 
 #include "lwasm.h"
+#include "instab.h"
 
 void write_code_raw(asmstate_t *as, FILE *of);
 void write_code_decb(asmstate_t *as, FILE *of);
@@ -43,6 +44,8 @@ void write_code_hex(asmstate_t *as, FILE *of);
 void write_code_srec(asmstate_t *as, FILE *of);
 void write_code_ihex(asmstate_t *as, FILE *of);
 void write_code_lwmod(asmstate_t *as, FILE *of);
+void write_code_dragon(asmstate_t *as, FILE *of);
+void write_code_abs(asmstate_t *as, FILE *of);
 
 // this prevents warnings about not using the return value of fwrite()
 // r++ prevents the "set but not used" warnings; should be optimized out
@@ -108,6 +111,14 @@ void do_output(asmstate_t *as)
 		write_code_lwmod(as, of);
 		break;
 
+	case OUTPUT_DRAGON:
+		write_code_dragon(as, of);
+		break;
+
+	case OUTPUT_ABS:
+		write_code_abs(as, of);
+		break;
+
 	default:
 		fprintf(stderr, "BUG: unrecognized output format when generating output file\n");
 		fclose(of);
@@ -120,7 +131,10 @@ void do_output(asmstate_t *as)
 
 int write_code_BASIC_fprintf(FILE *of, int linelength, int *linenumber, int value)
 {
-	if (linelength > 247)
+	// 240 should give enough room for a 5 digit value and a comma with a bit of extra
+	// space in case something unusual happens without going over the 249 character
+	// limit Color Basic has on input lines.
+	if (linelength > 240)
 	{
 		fprintf(of, "\n");
 		linelength = fprintf(of, "%d DATA ", *linenumber);
@@ -245,14 +259,26 @@ void write_code_rawrel(asmstate_t *as, FILE *of)
 raw merely writes all the bytes directly to the file as is. ORG is just a
 reference for the assembler to handle absolute references. Multiple ORG
 statements will produce mostly useless results
+
+However, if a run of RMBs exists at the start that is ended by an ORG
+statement, that run of RMBs will not be output as a zero fill.
 */
 void write_code_raw(asmstate_t *as, FILE *of)
 {
 	line_t *cl;
+	line_t *sl;
 	
-	for (cl = as -> line_head; cl; cl = cl -> next)
+	sl = as -> line_head;
+	for (cl = sl; cl; cl = cl -> next)
 	{
-		if (cl -> len > 0 && cl -> outputl == 0)
+		if (cl -> outputl > 0)
+			break;
+		if (instab[cl -> insn].flags & lwasm_insn_org)
+			sl = cl;
+	}
+	for (cl = sl; cl; cl = cl -> next)
+	{
+		if (cl -> len > 0 && cl -> outputl < 0)
 		{
 			int i;
 			for (i = 0; i < cl -> len; i++)
@@ -1243,3 +1269,94 @@ void write_code_lwmod(asmstate_t *as, FILE *of)
 	if (initsize)
 		writebytes(initcode, initsize, 1, of);
 }
+
+void write_code_abs_calc(asmstate_t *as, unsigned int *start, unsigned int *length)
+{
+	line_t *cl;
+	unsigned int lowaddr = 65535;
+	unsigned int highaddr = 0;
+	char outbyte;
+	int outaddr, rc;
+
+	// if not specified, calculate
+	for (cl = as -> line_head; cl; cl = cl -> next)
+	{
+		do
+		{
+			rc = fetch_output_byte(cl, &outbyte, &outaddr);
+			if (rc)
+			{
+				if (outaddr < lowaddr)
+					lowaddr = outaddr;
+				if (outaddr > highaddr)
+					highaddr = outaddr;
+			}
+		}
+		while (rc);
+	
+		*length = (lowaddr > highaddr) ? 0 : 1 + highaddr - lowaddr;
+		*start = (lowaddr > highaddr ) ? 0 : lowaddr;
+	}
+}
+
+void write_code_abs_aux(asmstate_t *as, FILE *of, unsigned int start, unsigned int header_size)
+{
+	line_t *cl;
+
+	char outbyte;
+	int outaddr, rc;
+
+	for (cl = as -> line_head; cl; cl = cl -> next)
+	{
+		do
+		{
+			rc = fetch_output_byte(cl, &outbyte, &outaddr);
+			
+			// if first byte to write or output stream jumps address, seek
+			if (rc == -1)
+			{
+				fseek(of,(long int) header_size + outaddr - start, SEEK_SET);
+			}
+			if (rc) fputc(outbyte,of);
+		}
+		while (rc);
+	}
+
+}
+
+/* Write a DragonDOS binary file */
+
+void write_code_dragon(asmstate_t *as, FILE *of)
+{
+	unsigned char headerbuf[9];
+	unsigned int start, length;
+
+	write_code_abs_calc(as, &start, &length);
+
+	headerbuf[0] = 0x55; // magic $55
+	headerbuf[1] = 0x02; // binary file
+	headerbuf[2] = (start >> 8) & 0xFF;
+	headerbuf[3] = (start) & 0xFF;
+	headerbuf[4] = (length >> 8) & 0xFF;
+	headerbuf[5] = (length) & 0xFF;
+	headerbuf[6] = (as -> execaddr >> 8) & 0xFF;
+	headerbuf[7] = (as -> execaddr) & 0xFF;
+	headerbuf[8] = 0xAA; // magic $AA
+
+	writebytes(headerbuf, 9, 1, of);
+
+	write_code_abs_aux(as, of, start, 9);
+
+}
+
+/* Write a monolithic binary block, respecting absolute address segments from ORG directives */
+/* Uses fseek, requires lowest code address and header offset size */
+/* Out of order ORG addresses are handled */
+
+void write_code_abs(asmstate_t *as, FILE *of)
+{
+	unsigned int start, length;
+	
+	write_code_abs_calc(as, &start, &length);
+	write_code_abs_aux(as, of, start, 0);
+}
