@@ -102,7 +102,7 @@ PARSEFUNC(pseudo_parse_end)
 
 	as->endseen = 1;
 
-	if ((as -> output_format != OUTPUT_DECB) && (as -> output_format != OUTPUT_BASIC) && (as -> output_format != OUTPUT_LWMOD && (as -> output_format != OUTPUT_IHEX) && (as -> output_format != OUTPUT_SREC)))
+	if ((as -> output_format != OUTPUT_DECB) && (as -> output_format != OUTPUT_BASIC) && (as -> output_format != OUTPUT_LWMOD) && (as -> output_format != OUTPUT_IHEX) && (as -> output_format != OUTPUT_SREC) && (as -> output_format != OUTPUT_DRAGON) && (as -> output_format != OUTPUT_ABS))
 	{
 		skip_operand(p);
 		return;
@@ -1111,7 +1111,8 @@ PARSEFUNC(pseudo_parse_ifp1)
 		return;
 	}
 	
-	lwasm_register_error2(as, l, W_NOT_SUPPORTED, "%s", "IFP1");
+	if ((as -> nowarn_flags & NOWARN_IFP1) == 0)
+		lwasm_register_error2(as, l, W_NOT_SUPPORTED, "%s", "IFP1");
 }
 
 PARSEFUNC(pseudo_parse_ifp2)
@@ -1127,7 +1128,8 @@ PARSEFUNC(pseudo_parse_ifp2)
 		return;
 	}
 	
-	lwasm_register_error2(as, l, W_NOT_SUPPORTED, "%s", "IFP2");
+	if ((as -> nowarn_flags & NOWARN_IFP1) == 0)
+		lwasm_register_error2(as, l, W_NOT_SUPPORTED, "%s", "IFP2");
 }
 
 PARSEFUNC(pseudo_parse_ifeq)
@@ -1340,6 +1342,12 @@ PARSEFUNC(pseudo_parse_ifdef)
 again:
 	for (i = 0; (*p)[i] && !isspace((*p)[i]) && (*p)[i] != '|' && (*p)[i] != '&'; i++)
 		/* do nothing */ ;
+
+	if (i == 0)
+	{
+		lwasm_register_error(as, l, E_OPERAND_BAD);
+		return;
+	}
 	
 	sym = lw_strndup(*p, i);
 	(*p) += i;
@@ -1380,6 +1388,12 @@ PARSEFUNC(pseudo_parse_ifndef)
 	for (i = 0; (*p)[i] && !isspace((*p)[i]) && (*p)[i] != '&' && (*p)[i] != '|'; i++)
 		/* do nothing */ ;
 	
+	if (i == 0)
+	{
+		lwasm_register_error(as, l, E_OPERAND_BAD);
+		return;
+	}
+
 	sym = lw_strndup(*p, i);
 	(*p) += i;
 	
@@ -1468,7 +1482,8 @@ PARSEFUNC(pseudo_parse_includebin)
 	FILE *fp;
 	long flen;
 	char *rfn;
-	
+	lw_expr_t e, e1;
+
 	if (!**p)
 	{
 		lwasm_register_error(as, l, E_FILENAME_MISSING);
@@ -1485,7 +1500,7 @@ PARSEFUNC(pseudo_parse_includebin)
 	}
 	else
 	{
-		for (p2 = *p; *p2 && !isspace(*p2); p2++)
+		for (p2 = *p; *p2 && *p2 != ',' && !isspace(*p2); p2++)
 			/* do nothing */ ;
 	}
 	fn = lw_strndup(*p, p2 - *p);
@@ -1507,13 +1522,99 @@ PARSEFUNC(pseudo_parse_includebin)
 	flen = ftell(fp);
 	fclose(fp);
 
-	l -> len = flen;
+	l -> lint2 = flen;
+
+	if (**p == ',')
+	{
+		(*p)++;
+		e = lwasm_parse_expr(as, p);
+
+		if (e)
+			lwasm_save_expr(l, 0, e);
+
+		if (**p == ',')
+		{
+			(*p)++;
+			e1 = lwasm_parse_expr(as, p);
+
+			if (e1)
+				lwasm_save_expr(l, 1, e1);
+		}
+	}
+	else
+	{
+		l -> len = flen; // length is resolved
+	}
+}
+
+RESOLVEFUNC(pseudo_resolve_includebin)
+{
+	lw_expr_t e, e1, n;
+	int i = 0, i1;
+
+	e = lwasm_fetch_expr(l, 0);
+	e1 = lwasm_fetch_expr(l, 1);
+	
+	// if offset and/or length specified, make sure they've resolved
+	// before we do anything
+	if (e && !lw_expr_istype(e, lw_expr_type_int))
+		return;
+	if (e1 && !lw_expr_istype(e, lw_expr_type_int))
+		return;
+	if (e != NULL)
+	{
+		i = lw_expr_intval(e);
+
+		if (i < 0)
+			i = l -> lint2 + i;
+	}
+	
+	i1 = l -> lint2 - i;
+
+	e1 = lwasm_fetch_expr(l, 1);
+
+	if (e1 != NULL)
+	{
+		i1 = lw_expr_intval(e1);
+	}
+
+	if( i < 0 )
+	{
+		/* starting before file */
+		lwasm_register_error(as, l, E_INCLUDEBIN_ILL_START);
+		return;
+	}
+
+	if (i > l -> lint2)
+	{
+		/* starts past end of file */
+		lwasm_register_error(as, l, E_INCLUDEBIN_ILL_START);
+		return;
+	}
+
+	if (i1 < 0)
+	{
+		/* length is negative */
+		lwasm_register_error(as, l, E_INCLUDEBIN_ILL_LENGTH);
+		return;
+	}
+
+	if (i + i1 > l -> lint2)
+	{
+		/* read past end of file */
+		lwasm_register_error(as, l, E_INCLUDEBIN_ILL_LENGTH);
+		return;
+	}
+
+
+	l -> lint = i; // pass forward offset
+	l -> len = i1;
 }
 
 EMITFUNC(pseudo_emit_includebin)
 {
 	FILE *fp;
-	int c;
+	int c, i;
 	
 	fp = fopen(l -> lstr, "rb");
 	if (!fp)
@@ -1522,7 +1623,10 @@ EMITFUNC(pseudo_emit_includebin)
 		return;
 	}
 	
-	for (;;)
+	// apply the specified offset
+	fseek(fp, l -> lint, SEEK_SET);
+
+	for (i=0; i < l -> len; i++)
 	{
 		c = fgetc(fp);
 		if (c == EOF)
