@@ -22,6 +22,8 @@ this program. If not, see <http://www.gnu.org/licenses/>.
 #ifndef ___lw_win_h_seen___
 #define ___lw_win_h_seen___
 
+#ifdef _MSC_VER
+
 #include "lw_string.h"
 #include <string.h>
 
@@ -29,8 +31,14 @@ this program. If not, see <http://www.gnu.org/licenses/>.
 #define strncasecmp _strnicmp
 #define unlink _unlink
 
+
+#if _MSC_VER < 1900
+// For older Microsoft stuff without snprintf
 int c99_snprintf(char* str, size_t size, const char* format, ...);
 
 #define snprintf c99_snprintf
+#endif
+
+#endif // _MSC_VER defined
 
 #endif /* ___lw_win_h_seen___ */
